@@ -1,4 +1,3 @@
-
 def make_table(headers, data):
     """
     headers [A, B]
@@ -7,12 +6,13 @@ def make_table(headers, data):
         (2, 3)
     ]
     """
-    txt = '|' +  '|'.join(headers) + '|\n' 
-    txt += '|---' * len(headers) + '|\n'
+    txt = "|" + "|".join(headers) + "|\n"
+    txt += "|---" * len(headers) + "|\n"
     for line in data:
-        txt += '|' +  '|'.join(line) + '|\n'
+        txt += "|" + "|".join(line) + "|\n"
     return txt
 
+
 def make_attr_table(data):
     """
     data [
@@ -24,7 +24,7 @@ def make_attr_table(data):
     headers = set()
     for _, attrs in data:
         headers = headers.union(attrs)
-    
+
     if headers:
         # Move "only" values to front.
         headers = list(headers)
@@ -32,23 +32,22 @@ def make_attr_table(data):
         left = []
         right = []
         for i in headers:
-            if 'only' in i.lower():
+            if "only" in i.lower():
                 left.append(i)
             else:
                 right.append(i)
         headers = left + right
-        
-        txt = '| |' +  '|'.join(headers) + '|\n' 
+
+        txt = "| |" + "|".join(headers) + "|\n"
     else:
-        txt = '| |\n'
-    txt += '|---' * (len(headers) + 1) + '|\n'
+        txt = "| |\n"
+    txt += "|---" * (len(headers) + 1) + "|\n"
     for obj, attrs in data:
         txt += "|" + obj
         for attr in headers:
             if attr in attrs:
-                txt += '| [✓] '
+                txt += "| [✓] "
             else:
-                txt += '| '
+                txt += "| "
         txt += "|\n"
     return txt
-
