@@ -1,19 +1,25 @@
 import json
 import os
 import re
-from typing import Dict, Optional, Union
 from json import JSONDecodeError
+from typing import Dict, Optional, Union
 
-from crytic_compile.platform import solc_standard_json
-from crytic_compile.crytic_compile import CryticCompile
 from crytic_compile.compilation_unit import CompilationUnit
 from crytic_compile.compiler.compiler import CompilerVersion
-from crytic_compile.platform.etherscan import Etherscan, _convert_version, _handle_multiple_files, _handle_single_file
+from crytic_compile.crytic_compile import CryticCompile
+from crytic_compile.platform import solc_standard_json
+from crytic_compile.platform.etherscan import (
+    Etherscan,
+    _convert_version,
+    _handle_multiple_files,
+    _handle_single_file,
+)
 from crytic_compile.platform.exceptions import InvalidCompilation
 from slither.slither import Slither
 
+from peth.core.config import config
 from peth.eth.scan import ScanAPI
-from peth.core import config
+
 
 class Uniscan(Etherscan):
 
@@ -25,24 +31,24 @@ class Uniscan(Etherscan):
 
         target = self._target
         export_dir = os.path.join(
-            config.OUTPUT_PATH, kwargs.get("etherscan_export_dir", "compile-contracts")
+            config.output_path, kwargs.get("etherscan_export_dir", "compile-contracts")
         )
 
         source_code: str = ""
         result: Dict[str, Union[bool, str, int]] = {}
         contract_name: str = ""
 
-        chain, addr = target.split(':')
+        chain, addr = target.split(":")
         result = ScanAPI.get_or_create_by_chain(chain).get_contract_info(addr)
 
         if not result:
             raise InvalidCompilation("Contract has no public source code: " + target)
 
-        prefix = '-' + chain
+        prefix = "-" + chain
         # Assert to help mypy
         assert isinstance(result["SourceCode"], str)
         assert isinstance(result["ContractName"], str)
-        
+
         source_code = result["SourceCode"]
         contract_name = result["ContractName"]
 
@@ -80,7 +86,9 @@ class Uniscan(Etherscan):
                 )
             except JSONDecodeError:
                 filenames = [
-                    _handle_single_file(source_code, addr, prefix, contract_name, export_dir)
+                    _handle_single_file(
+                        source_code, addr, prefix, contract_name, export_dir
+                    )
                 ]
 
         compilation_unit = CompilationUnit(crytic_compile, contract_name)
@@ -93,11 +101,14 @@ class Uniscan(Etherscan):
         )
         compilation_unit.compiler_version.look_for_installed_version()
 
-        solc_standard_json.standalone_compile(filenames, compilation_unit, working_dir=working_dir)
+        solc_standard_json.standalone_compile(
+            filenames, compilation_unit, working_dir=working_dir
+        )
+
 
 def slither_from_chain_addr(chain, addr):
     target = f"{chain}:{addr}"
     platform = Uniscan(target)
     compile = CryticCompile(platform)
     s = Slither(compile)
-    return s
\ No newline at end of file
+    return s
