@@ -1,29 +1,34 @@
 import difflib
 import os
 
-from peth.core import config
+from peth.core.config import config
 
 _disable_format = False
+
+
 def format_solidity(code):
     global _disable_format
 
     if _disable_format:
         return code
 
-    if not os.path.exists(config.DIFF_PATH):
-        os.makedirs(config.DIFF_PATH)
+    if not os.path.exists(config.diff_path):
+        os.makedirs(config.diff_path)
 
-    path = config.DIFF_TMP_FILE
+    path = os.path.join(config.diff_path, "peth.tmp.sol")
     open(path, "w").write(code)
 
-    ret = os.system('npx prettier -w %s >/dev/null' % path)
+    ret = os.system("npx prettier -w %s >/dev/null" % path)
     if ret != 0:
-        print("Run `npm install -g prettier prettier-plugin-solidity` to install solidity formatter to get better diff result.")
+        print(
+            "Run `npm install -g prettier prettier-plugin-solidity` to install solidity formatter to get better diff result."
+        )
         _disable_format = True
         return code
 
     return open(path).read()
 
+
 class ContractSource(object):
 
     def __init__(self, code=None) -> None:
@@ -31,7 +36,6 @@ class ContractSource(object):
         if code:
             self.parse_source(code)
 
-
     def parse_source(self, code: str):
         """
         Split full source code into contracts code.
@@ -43,35 +47,35 @@ class ContractSource(object):
             # Find the line which starts with "contract"
 
             # TODO: abstract contract, interface ?
-            if not code.startswith('contract') and not code.startswith('library'):
-                i = code.find('\ncontract')
-                i2 = code.find('\nlibrary')
-                if i != -1 and i2 != -1 and i2 < i: # Search the nearest one.
+            if not code.startswith("contract") and not code.startswith("library"):
+                i = code.find("\ncontract")
+                i2 = code.find("\nlibrary")
+                if i != -1 and i2 != -1 and i2 < i:  # Search the nearest one.
                     i = i2
 
                 if i == -1:
                     # Not contract here.
                     break
-                code = code[i + 1:]  # skip \n
+                code = code[i + 1 :]  # skip \n
 
             # Find the end of contract.
             count = 0
             end = 0
             for i, c in enumerate(code):
-                if c == '{':
+                if c == "{":
                     count += 1
 
-                if c == '}':
+                if c == "}":
                     count -= 1
                     assert count >= 0, "Curly brackets do NOT match"
                     if count == 0:
                         end = i + 1  # skip }
                         break
 
-            contract_code = code[: end]
+            contract_code = code[:end]
 
             # format the code.
-            contract_code = format_solidity(contract_code) 
+            contract_code = format_solidity(contract_code)
 
             contract_name = contract_code[:100].split()[1]
 
@@ -84,13 +88,13 @@ class ContractSource(object):
     def __diff_file(self, a, b, output_filename):
         d = difflib.HtmlDiff()
         buf = d.make_file(a.splitlines(), b.splitlines())
-        open(output_filename + '.html', 'w').write(buf)
-        print("Written to " + output_filename+'.html')
+        open(output_filename + ".html", "w").write(buf)
+        print("Written to " + output_filename + ".html")
 
     def compare(self, other, output=None):
 
         if output is None:
-            output = config.DIFF_PATH
+            output = config.diff_path
 
         if not os.path.isdir(output):
             os.makedirs(output)
@@ -99,7 +103,10 @@ class ContractSource(object):
         dst_left = dict(other.contracts)
 
         to_comp = dict(other.contracts)
-        for name1, src1, in self.contracts.items():
+        for (
+            name1,
+            src1,
+        ) in self.contracts.items():
             if name1 in to_comp:
                 src2 = to_comp[name1]
                 s = difflib.SequenceMatcher(None, src1.splitlines(), src2.splitlines())
@@ -111,25 +118,22 @@ class ContractSource(object):
 
                 # Skip similarity-guided comparison for the contract.
                 del to_comp[name1]
-                continue 
+                continue
 
             for name2, src2 in to_comp.items():
                 s = difflib.SequenceMatcher(None, src1.splitlines(), src2.splitlines())
                 similarity = s.ratio()
                 filename = "%s_%s_%0.2f" % (name1, name2, similarity)
                 # print(filename)
-                if similarity > config.DIFF_MIN_SIMILARITY:
+                if similarity > config.diff_min_similarity:
                     self.__diff_file(src1, src2, os.path.join(output, filename))
                     src_left[name1] = None
                     dst_left[name2] = None
-        
+
         src_left_names = list(filter(lambda x: src_left[x], src_left))
         dst_left_names = list(filter(lambda x: dst_left[x], dst_left))
         if src_left_names or dst_left_names:
             print("Non-matched contracts:")
-            print(','.join(src_left_names))
-            print('-'*10)
-            print(','.join(dst_left_names))
-
-        
-
+            print(",".join(src_left_names))
+            print("-" * 10)
+            print(",".join(dst_left_names))
